#ifndef KUBAS_AVRLIB_USART0
#define KUBAS_AVRLIB_USART0

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/wdt.h>
#include "queue.h"

namespace kubas
{

#ifndef USART_RX_BUF
	#if defined (__AVR_ATmega48__)
		#define UART_RX_BUF 32
	#else
		#define UART_RX_BUF 96
	#endif
#endif
#ifndef USART_TX_BUF
	#if defined (__AVR_ATmega48__)
		#define USART_TX_BUF 32
	#else
		#define USART_TX_BUF 96
	#endif
#endif


#if ((defined (__AVR_ATmega16__))||
	 (defined (__AVR_ATmega32__)))
		#define ___UCSRA	UCSRA
		#define ___UCSRB	UCSRB
		#define ___UCSRC	UCSRC
		#define ___UBRRL	UBRRL
		#define ___UBRRH	UBRRH
		#define ___UDR		UDR
		#define ___RXC		(1<<RXC)
		#define ___TXC		(1<<TXC)
		#define ___UDRE		(1<<UDRE)
		#define ___FE		(1<<FE)
		#define ___DOR		(1<<DOR)
		#define ___PE		(1<<PE)
		#define ___U2X		(1<<U2X)
		#define ___MPCM		(1<<MPCM)
		#define ___RXCIE	(1<<RXCIE)
		#define ___TXCIE	(1<<TXCIE)
		#define ___UDRIE	(1<<UDRIE)
		#define ___RXEN		(1<<RXEN)
		#define ___TXEN		(1<<TXEN)
		#define ___UCSZ2	(1<<UCSZ2)
		#define ___RX8B		(1<<RX8B)
		#define ___TX8B		(1<<TX8B)
		#define ___URSEL	(1<<URSEL)
		#define ___UMSEL	(1<<UMSEL)
		#define ___UPM1		(1<<UPM1)
		#define ___UPM0 	(1<<UPM0)
		#define ___USBS		(1<<USBS)
		#define ___UCSZ1	(1<<UCSZ1)
		#define ___UCSZ0	(1<<UCSZ0)
		#define ___UCPOL	(1<<UCPOL)
		#define ___RXC_INT	USART_RXC_vect
		#define ___TXC_INT	USART_TXC_vect
		#define ___UDRE_INT	USART_UDRE_vect
#elif ((defined _AVR_IOMX8_H_)||
	   (defined(__AVR_ATmega128__))||
	   (defined (_AVR_IOM1284P_H_)))
	    #define ___UCSRA	UCSR0A
		#define ___UCSRB	UCSR0B
		#define ___UCSRC	UCSR0C
		#define ___UBRRL	UBRR0L
		#define ___UBRRH	UBRR0H
		#define ___UDR		UDR0
		#define ___RXC		(1<<RXC0)
		#define ___TXC		(1<<TXC0)
		#define ___UDRE		(1<<UDRE0)
		#define ___FE		(1<<FE0)
		#define ___DOR		(1<<DOR0)
		#define ___PE		(1<<PE0)
		#define ___U2X		(1<<U2X0)
		#define ___MPCM		(1<<MPCM0)
		#define ___RXCIE	(1<<RXCIE0)
		#define ___TXCIE	(1<<TXCIE0)
		#define ___UDRIE	(1<<UDRIE0)
		#define ___RXEN		(1<<RXEN0)
		#define ___TXEN		(1<<TXEN0)
		#define ___UCSZ2	(1<<UCSZ02)
		#define ___RX8B		(1<<RX8B0)
		#define ___TX8B		(1<<TX8B0)
		#define ___URSEL	(1<<URSEL0)
		#define ___UMSEL	(1<<UMSEL0)
		#define ___UPM1		(1<<UPM01)
		#define ___UPM0 	(1<<UPM00)
		#define ___USBS		(1<<USBS0)
		#define ___UCSZ1	(1<<UCSZ01)
		#define ___UCSZ0	(1<<UCSZ00)
		#define ___UCPOL	(1<<UCPOL0)
	   #if ((defined(__AVR_ATmega128__))||(defined (_AVR_IOM1284P_H_)))
		#define ___RXC_INT	USART_RXC0_vect
		#define ___TXC_INT	USART_TXC0_vect
		#define ___UDRE_INT	USART_UDRE0_vect
	   #elif
	    #define ___RXC_INT	USART_RXC0_vect
		#define ___TXC_INT	USART_TXC0_vect
		#define ___UDRE_INT	USART_UDRE0_vect
	   #endif
#endif

template <typename Usart, int RxBufferSize = USART_RX_BUF, int TxBufferSize = USART_TX_BUF, typename Bootseq = nobootseq, typename Overflow = uint32_t>
class usart_t {

#ifndef USART_EXTERN_RX_INT
	queue <char, RxBufferSize> m_rx;
#endif
#ifndef USART_EXTERN_TX_INT
	queue <char, TxBufferSize> m_tx;
#endif

public:

	inline void data_in(const char ch) { m_rx.push(ch); }

	inline bool data_out(const char &ch)
	{
		if(m_tx.is_empty())
			return false;
		ch = m_tx.pop();
		return true;
	}

	inline bool is_send() { return m_tx.is_empty();	}

	void init(uint32_t speed)
	{
		___UCSRA = ___U2X;
		___UCSRB = ___RXCIE | ___RXEN | ___TXEN;
		___UCSRC = ___URSEL | ___UCSZ0 | ___UCSZ1;
		speed = uint16_t(F_CPU / (8 * speed) - 1);
		___UBRRH = ((speed&0xFF00)>>8);
		___UBRRL = (speed&0x00FF);
		#ifdef USART_DIR_CTRL_PIN
			USART_DIR_CTRL_PIN::output(true);
			USART_DIR_CTRL_PIN::clear();
			___UCSRB |= ___TXCIE;
		#endif
	}

	inline void stop() { ___UCSRB = 0; }

	void write(char data)
	{
		while(!m_tx.push(data)) {}
		#ifdef USART_DIR_CTRL_PIN
			USART_DIR_CTRL_PIN::set();
		#endif
		___UCSRB |= ___UDRIE;
	}
	
	void write_immediately(char data)
	{
		#ifdef USART_DIR_CTRL_PIN
			USART_DIR_CTRL_PIN::set();
		#endif
		___UCSRA |= ___TXC;
		___UDR = data;
		while((___UCSRA & ___TXC) == 0) {}
		#ifdef USART_DIR_CTRL_PIN
			USART_DIR_CTRL_PIN::clear();
		#endif
	}

	inline void wait() { while(!m_tx.is_empty()) {}	}

	bool peek(char & data)
	{
		if(m_rx.is_empty())
			return false;
		data = m_rx.pop();
		return true;
	}

	char get()
	{
		char data;
		while(!peek(data)) {}
		return data;
	}

	void process_rx()
	{
		char data = UDR;
		if((___UCSRA & __FE) != 0)
			return;
		data_in(data);
	}

	void process_tx()
	{
		char data;
		if(data_out(data))
			UDR = data;
		else
			___UCSRB &= ~___UDRIE;
	}

	#ifdef USART_DIR_CTRL_PIN
		void process_txe() { USART_CTRL_PIN::clear(); }
	#endif

}; usart0_t usart0;

}//end of namespace
#endif